window.Gleam.initSearch([{"doc":"mist","title":"mist","content":"# mist\n\nA glistening Gleam web server.\n\n## Installation\n\nThis package can be added to your Gleam project:\n\n```sh\ngleam add mist\n```\n\nand its documentation can be found at &lt;https://hexdocs.pm/mist&gt;.\n\n## Usage\n\nThe main entrypoints for your application are `mist.start_http` and\n`mist.start_https`. The argument to these functions is generated from the\nopaque `Builder` type. It can be constructed with the `mist.new` function, and\nfed updated configuration options with the associated methods (demonstrated\nin the examples below).\n\n```gleam\nimport gleam/bit_builder\nimport gleam/bit_string\nimport gleam/erlang/process\nimport gleam/http/request.{Request}\nimport gleam/http/response.{Response}\nimport gleam/iterator\nimport gleam/otp/actor\nimport gleam/result\nimport gleam/string\nimport mist/file\nimport mist.{Connection, ResponseData}\n\npub fn main() {\n  // These values are for the Websocket process initialized below\n  let selector = process.new_selector()\n  let state = Nil\n\n  let not_found =\n    response.new(404)\n    |&gt; response.set_body(mist.Bytes(bit_builder.new()))\n\n  fn(req: Request(Connection)) -&gt; Response(ResponseData) {\n    case request.path_segments(req) {\n      [&quot;ws&quot;] -&gt;\n        mist.websocket(req)\n        |&gt; mist.with_state(state)\n        |&gt; mist.selecting(selector)\n        |&gt; mist.on_message(handle_ws_message)\n        |&gt; mist.upgrade\n      [&quot;echo&quot;] -&gt; echo_body(req)\n      [&quot;chunk&quot;] -&gt; serve_chunk(req)\n      [&quot;file&quot;, ..rest] -&gt; serve_file(req, rest)\n      [&quot;form&quot;] -&gt; handle_form(req)\n\n      _ -&gt; not_found\n    }\n  }\n  |&gt; mist.new\n  |&gt; mist.port(8080)\n  |&gt; mist.start_http\n\n  process.sleep_forever()\n}\n\npub type MyMessage {\n  Broadcast(String)\n}\n\nfn handle_ws_message(state, conn, message) {\n  case message {\n    mist.Text(&lt;&lt;&quot;ping&quot;:utf8&gt;&gt;) -&gt; {\n      let assert Ok(_) = mist.send_text_frame(conn, &lt;&lt;&quot;pong&quot;:utf8&gt;&gt;)\n      actor.Continue(state)\n    }\n    mist.Text(_) | mist.Binary(_) -&gt; {\n      actor.Continue(state)\n    }\n    mist.Custom(Broadcast(text)) -&gt; {\n      let assert Ok(_) = mist.send_text_frame(conn, &lt;&lt;text:utf8&gt;&gt;)\n      actor.Continue(state)\n    }\n    mist.Closed | mist.Shutdown -&gt; actor.Stop(process.Normal)\n  }\n}\n\nfn echo_body(request: Request(Connection)) -&gt; Response(ResponseData) {\n  let content_type =\n    request\n    |&gt; request.get_header(&quot;content-type&quot;)\n    |&gt; result.unwrap(&quot;text/plain&quot;)\n\n  mist.read_body(request, 1024 * 1024 * 10)\n  |&gt; result.map(fn(req) {\n    response.new(200)\n    |&gt; response.set_body(mist.Bytes(bit_builder.from_bit_string(req.body)))\n    |&gt; response.set_header(&quot;content-type&quot;, content_type)\n  })\n  |&gt; result.lazy_unwrap(fn() {\n    response.new(400)\n    |&gt; response.set_body(mist.Bytes(bit_builder.new()))\n  })\n}\n\nfn serve_chunk(_request: Request(Connection)) -&gt; Response(ResponseData) {\n  let iter =\n    [&quot;one&quot;, &quot;two&quot;, &quot;three&quot;]\n    |&gt; iterator.from_list\n    |&gt; iterator.map(bit_builder.from_string)\n\n  response.new(200)\n  |&gt; response.set_body(mist.Chunked(iter))\n  |&gt; response.set_header(&quot;content-type&quot;, &quot;text/plain&quot;)\n}\n\nfn serve_file(\n  _req: Request(Connection),\n  path: List(String),\n) -&gt; Response(ResponseData) {\n  let not_found =\n    response.new(404)\n    |&gt; response.set_body(mist.Bytes(bit_builder.new()))\n\n  let file_path =\n    path\n    |&gt; string.join(&quot;/&quot;)\n    |&gt; bit_string.from_string\n\n  // Omitting validation for brevity\n  let file_descriptor = file.open(file_path)\n  let file_size = file.size(file_path)\n\n  case file_descriptor {\n    Ok(file) -&gt; {\n      let content_type = guess_content_type(file_path)\n      response.new(200)\n      |&gt; response.set_body(mist.File(file, content_type, 0, file_size))\n    }\n    _ -&gt; not_found\n  }\n}\n\nfn handle_form(req: Request(Connection)) -&gt; Response(ResponseData) {\n  let _req = mist.read_body(req, 1024 * 1024 * 30)\n  response.new(200)\n  |&gt; response.set_body(mist.Bytes(bit_builder.new()))\n}\n\nfn guess_content_type(_path: BitString) -&gt; String {\n  &quot;application/octet-stream&quot;\n}\n```\n\n## Streaming request body\n\n```\nNOTE:  This is a new feature, and I may have made some mistakes.  Please let me\nknow if you run into anything :)\n```\n\nWhen handling file uploads or `multipart/form-data`, you probably don&#39;t want to\nload the whole file into memory. Previously, the only options in `mist` were to\naccept bodies up to `N` bytes, or read the entire body.\n\nNow, there is a `mist.stream` function which takes a `Request(Connection)` that\ngives you back a function to start reading chunks. This function will return:\n\n```gleam\npub type Chunk {\n  Chunk(data: BitString, consume: fn(Int) -&gt; Chunk)\n  Done\n}\n```\n\nNOTE: You must only call this once on the `Request(Connection)`. Since it&#39;s\nreading data from the socket, this is a mutable action. The name `consume` was\nchosen to hopefully make that more clear.\n\n### Example\n\n```gleam\n// Replacing the named function in the application example above\nfn handle_form(req: Request(Connection)) -&gt; Response(ResponseData) {\n  let assert Ok(consume) = mist.stream(req)\n  // NOTE:  This is a little misleading, since `Iterator`s can be replayed.\n  // However, this will only be running this once.\n  let content =\n    iterator.unfold(\n      consume,\n      fn(consume) {\n        // Reads up to 1024 bytes from the request\n        let res = consume(1024)\n        case res {\n          // The error will not be bubbled up to the iterator here. If either\n          // we&#39;ve read all the body, or we see an error, the iterator finishes\n          Ok(mist.Done) | Error(_) -&gt; iterator.Done\n          // We read some data. It may be less than the specific amount above if\n          // we have consumed all of the body. You&#39;ll still need to call it\n          // again to ensure, since with `chunked` encoding, we need to check\n          // for the last chunk.\n          Ok(mist.Chunk(data, consume)) -&gt; {\n            iterator.Next(bit_builder.from_bit_string(data), consume)\n          }\n        }\n      },\n    )\n  // For fun, respond with `chunked` encoding of the same iterator\n  response.new(200)\n  |&gt; response.set_body(mist.Chunked(content))\n}\n```\n","url":"index.html"},{"doc":"mist","title":"Builder","content":"pub opaque type Builder(request_body, response_body)\n\n\nSynonyms:\nmist.Builder\nmist Builder","url":"mist.html#Builder"},{"doc":"mist","title":"Chunk","content":"pub type Chunk {\n  Chunk(\n    data: BitString,\n    consume: fn(Int) -&gt; Result(Chunk, ReadError),\n  )\n  Done\n}\n The values returning from streaming the request body. The `Chunk`\n variant gives back some data and the next token. `Done` signifies\n that we have completed reading the body.\nChunk(\n  data: BitString,\n  consume: fn(Int) -&gt; Result(Chunk, ReadError),\n)\n\n\nDone\n\n\nSynonyms:\nmist.Chunk\nmist Chunk","url":"mist.html#Chunk"},{"doc":"mist","title":"Connection","content":"pub type Connection =\n  InternalConnection\n Re-exported type that represents the default `Request` body type. See\n `mist.read_body` to convert this type into a `BitString`. The `Connection`\n also holds some additional information about the request. Currently, the\n only useful field is `client_ip` which is a `Result` with a tuple of\n integers representing the IPv4 address.\n\nSynonyms:\nmist.Connection\nmist Connection","url":"mist.html#Connection"},{"doc":"mist","title":"ReadError","content":"pub type ReadError {\n  ExcessBody\n  MalformedBody\n}\n The possible errors from reading the request body. If the size is larger\n than the provided value, `ExcessBody` is returned. If there is an error\n reading the body from the socket or the body is malformed (i.e a chunked\n request with invalid sizes), `MalformedBody` is returned.\nExcessBody\n\n\nMalformedBody\n\n\nSynonyms:\nmist.ReadError\nmist ReadError","url":"mist.html#ReadError"},{"doc":"mist","title":"ResponseData","content":"pub type ResponseData {\n  Websocket(Selector(ProcessDown))\n  Bytes(BitBuilder)\n  Chunked(Iterator(BitBuilder))\n  File(\n    descriptor: file.FileDescriptor,\n    content_type: String,\n    offset: Int,\n    length: Int,\n  )\n}\n The response body type. This allows `mist` to handle these different cases\n for you. `Bytes` is the regular data return. `Websocket` will upgrade the\n socket to websockets, but should not be used directly. See the\n `mist.upgrade` function for usage. `Chunked` will use\n `Transfer-Encoding: chunked` to send an iterator in chunks. `File` will use\n Erlang&#39;s `sendfile` to more efficiently return a file to the client. See the\n `mist.file` module for some helper functions.\nWebsocket(Selector(ProcessDown))\n\n\nBytes(BitBuilder)\n\n\nChunked(Iterator(BitBuilder))\n\n\nFile(\n  descriptor: file.FileDescriptor,\n  content_type: String,\n  offset: Int,\n  length: Int,\n)\n\n\nSynonyms:\nmist.ResponseData\nmist ResponseData","url":"mist.html#ResponseData"},{"doc":"mist","title":"WebsocketBuilder","content":"pub opaque type WebsocketBuilder(state, message)\n\n\nSynonyms:\nmist.WebsocketBuilder\nmist WebsocketBuilder","url":"mist.html#WebsocketBuilder"},{"doc":"mist","title":"WebsocketMessage","content":"pub type WebsocketMessage(custom) {\n  Text(BitString)\n  Binary(BitString)\n  Closed\n  Shutdown\n  Custom(custom)\n}\n These are the types of messages that a websocket handler may receive.\nText(BitString)\n\n\nBinary(BitString)\n\n\nClosed\n\n\nShutdown\n\n\nCustom(custom)\n\n\nSynonyms:\nmist.WebsocketMessage\nmist WebsocketMessage","url":"mist.html#WebsocketMessage"},{"doc":"mist","title":"after_start","content":"pub fn after_start(builder: Builder(a, b), after_start: fn(Int) -&gt;\n    Nil) -&gt; Builder(a, b)\n Override the default function to be called after the service starts. The\n default is to log a message with the listening port.\nSynonyms:\nmist.after_start\nmist after_start","url":"mist.html#after_start"},{"doc":"mist","title":"new","content":"pub fn new(handler: fn(Request(a)) -&gt; Response(b)) -&gt; Builder(\n  a,\n  b,\n)\n Create a new `mist` handler with a given function. The default port is\n 4000.\nSynonyms:\nmist.new\nmist new","url":"mist.html#new"},{"doc":"mist","title":"on_message","content":"pub fn on_message(builder: WebsocketBuilder(a, b), handler: fn(\n    a,\n    WebsocketConnection,\n    WebsocketMessage(b),\n  ) -&gt; Next(a)) -&gt; WebsocketBuilder(a, b)\n Provides a function to call for each `WebsocketMessage` received by the\n process.\nSynonyms:\nmist.on_message\nmist on_message","url":"mist.html#on_message"},{"doc":"mist","title":"port","content":"pub fn port(builder: Builder(a, b), port: Int) -&gt; Builder(a, b)\n Assign a different listening port to the service.\nSynonyms:\nmist.port\nmist port","url":"mist.html#port"},{"doc":"mist","title":"read_body","content":"pub fn read_body(req: Request(Connection), max_body_limit max_body_limit: Int) -&gt; Result(\n  Request(BitString),\n  ReadError,\n)\n The request body is not pulled from the socket until requested. The\n `content-length` header is used to determine whether the socket is read\n from or not. The read may also fail, and a `ReadError` is raised.\nSynonyms:\nmist.read_body\nmist read_body","url":"mist.html#read_body"},{"doc":"mist","title":"read_request_body","content":"pub fn read_request_body(builder: Builder(BitString, a), bytes_limit bytes_limit: Int, failure_response failure_response: Response(\n    a,\n  )) -&gt; Builder(Connection, a)\n This function allows for implicitly reading the body of requests up\n to a given size. If the size is too large, or the read fails, the provided\n `failure_response` will be sent back as the response.\nSynonyms:\nmist.read_request_body\nmist read_request_body","url":"mist.html#read_request_body"},{"doc":"mist","title":"selecting","content":"pub fn selecting(builder: WebsocketBuilder(a, b), selector: Selector(\n    b,\n  )) -&gt; WebsocketBuilder(a, b)\n Provide an external selector for user-specified messages that the websocket\n process may receive. These will be provided in the `Custom` type.\nSynonyms:\nmist.selecting\nmist selecting","url":"mist.html#selecting"},{"doc":"mist","title":"send_binary_frame","content":"pub fn send_binary_frame(connection: WebsocketConnection, frame: BitString) -&gt; Result(\n  Nil,\n  SocketReason,\n)\n Sends a binary frame across the websocket.\nSynonyms:\nmist.send_binary_frame\nmist send_binary_frame","url":"mist.html#send_binary_frame"},{"doc":"mist","title":"send_text_frame","content":"pub fn send_text_frame(connection: WebsocketConnection, frame: BitString) -&gt; Result(\n  Nil,\n  SocketReason,\n)\n Sends a text frame across the websocket.\nSynonyms:\nmist.send_text_frame\nmist send_text_frame","url":"mist.html#send_text_frame"},{"doc":"mist","title":"start_http","content":"pub fn start_http(builder: Builder(Connection, ResponseData)) -&gt; Result(\n  Nil,\n  StartError,\n)\n Start a `mist` service over HTTP with the provided builder.\nSynonyms:\nmist.start_http\nmist start_http","url":"mist.html#start_http"},{"doc":"mist","title":"start_https","content":"pub fn start_https(builder: Builder(Connection, ResponseData), certfile certfile: String, keyfile keyfile: String) -&gt; Result(\n  Nil,\n  StartError,\n)\n Start a `mist` service over HTTPS with the provided builder. This method\n requires both a certificate file and a key file. The library will attempt\n to read these files off of the disk.\nSynonyms:\nmist.start_https\nmist start_https","url":"mist.html#start_https"},{"doc":"mist","title":"stream","content":"pub fn stream(req: Request(Connection)) -&gt; Result(\n  fn(Int) -&gt; Result(Chunk, ReadError),\n  ReadError,\n)\n Rather than explicitly reading either the whole body (optionally up to\n `N` bytes), this function allows you to consume a stream of the request\n body. Any errors reading the body will propagate out, or `Chunk`s will be\n emitted. This provides a `consume` method to attempt to grab the next\n `size` chunk from the socket.\nSynonyms:\nmist.stream\nmist stream","url":"mist.html#stream"},{"doc":"mist","title":"upgrade","content":"pub fn upgrade(builder: WebsocketBuilder(a, b)) -&gt; Response(\n  ResponseData,\n)\n Upgrade a request to handle websockets. If the request is\n malformed, or the websocket process fails to initialize, an empty\n 400 response will be sent to the client.\nSynonyms:\nmist.upgrade\nmist upgrade","url":"mist.html#upgrade"},{"doc":"mist","title":"websocket","content":"pub fn websocket(request: Request(Connection)) -&gt; WebsocketBuilder(\n  Nil,\n  a,\n)\n Initializes a builder for upgrading a connection to Websockets. The\n default handler will shut down the process on receipt of a message.\n The default state is empty, and no external messages can be received.\nSynonyms:\nmist.websocket\nmist websocket","url":"mist.html#websocket"},{"doc":"mist","title":"with_state","content":"pub fn with_state(builder: WebsocketBuilder(a, b), state: a) -&gt; WebsocketBuilder(\n  a,\n  b,\n)\n Adds some initial state to the websocket handler.\nSynonyms:\nmist.with_state\nmist with_state","url":"mist.html#with_state"},{"doc":"mist","title":"mist","content":"","url":"mist.html"},{"doc":"mist/file","title":"FileError","content":"pub type FileError {\n  IsDir\n  NoAccess\n  NoEntry\n  UnknownFileError\n}\n Errors returned from attempting to open a file.\nIsDir\n\n\nNoAccess\n\n\nNoEntry\n\n\nUnknownFileError\n\n\nSynonyms:\nmist/file.FileError\nmist/file FileError","url":"mist/file.html#FileError"},{"doc":"mist/file","title":"open","content":"pub fn open(file: BitString) -&gt; Result(FileDescriptor, FileError)\n Attemps to open a file at the given path. Returns a `FileDescriptor`\n to use with the `mist.File` response type.\nSynonyms:\nmist/file.open\nmist/file open","url":"mist/file.html#open"},{"doc":"mist/file","title":"size","content":"pub fn size(path: BitString) -&gt; Int\n Get the file size from a path. Returns `0` if the path\n does not exist.\nSynonyms:\nmist/file.size\nmist/file size","url":"mist/file.html#size"},{"doc":"mist/file","title":"mist/file","content":"","url":"mist/file.html"}]);